function [attrib_feature, TT] = attrib_filter(image, attribute, threshold, dual)
%ATTRIB_FILTER filters a grayscale image by removing all objects for which
%a certain attribute of choice does not meet a predefined threshold
%criterion.
%
%   ATTRIB_FEATURE = ATTRIB_FILTER(IMAGE, ATTRIBUTE, THRESHOLD, DUAL)
%   thresholds the input image on each of its gray values, resulting in a
%   set of binary images. For each of these binary images, objects are
%   considered, i.e. groups of pixels that are connected according to a
%   certain connectivity rule. In this function, 8-connectivity is used.
%   For each of the objects, an attribute is considered, for instance
%   'area', 'moment of inertia' or 'standard deviation'. In each of the
%   binary images, all objects are removed that do not satisfy a certain
%   predefined threshold. Finally, a filtered grayscale image is
%   reconstructed by assigning to each pixel the maximum gray value,
%   corresponding to the order of the binary images, for which the
%   criterion is still satisfied.
%
%   ATTRIB_FEATURE is a filtered grayscale image of the same size as the
%   input image.
%
%   IMAGE is the input grayscale image. Gray values should be discrete!
%   ATTRIBUTE is type of the attribute that is considered. Possible values
%   are 'area', 'mi' and 'sd, corresponding to 'area', 'moment of inertia'
%   and 'standard deviation', respectively.
%   THRESHOLD is the threshold criterion for the selected attribute.
%   DUAL (optional) selects between opening (default) and closing
%   (thickening or thinning), which are dual operations. 

if ~exist('dual','var')
    dual = 'open';
end
MGV=max(image(:));
switch dual
    case 'close'
        image = MGV - image;
    otherwise
end

[r,c] = size(image);
grayvalue = unique(image(:));
numofgrayvalues = length(grayvalue);
Ilayers = zeros(r,c,2);
TT=zeros(r,c,numofgrayvalues);
for i = 1:numofgrayvalues
    
    Igi = image;
    Igi(Igi < grayvalue(i)) = 0;
    Igi(Igi >= grayvalue(i)) = 1;
    cc = bwconncomp(Igi);
    
    switch attribute
        case 'area'
            areacc = regionprops(cc,'Area');
            idx = find([areacc.Area] > threshold);
            if any(idx)
                BW2 = ismember(labelmatrix(cc), idx);
                Ilayers(:,:,2) = BW2*grayvalue(i);
                TT(:,:,i)=BW2*grayvalue(i);
            end
        case 'mi'
            imagecc = regionprops(cc,'Image');
            len = length(imagecc);
            mi_reg = false(len,1);
            for k = 1:len
                mom_inertia = moments(imagecc(k).Image);
                if(mom_inertia > threshold)
                    mi_reg(k) = true;
                end
            end
            if any(mi_reg)
                BW2 = ismember(labelmatrix(cc), find(mi_reg));
                Ilayers(:,:,2) = BW2*grayvalue(i);
            end
            
        case 'sd'
            pixelidcc = regionprops(cc,'PixelIdxList');
            len = length(pixelidcc);
            sd_reg = false(len,1);
            for k = 1:len
                pixelsid = double(image(pixelidcc(k).PixelIdxList));
                stdarea = std(pixelsid);
                if(stdarea > threshold)
                    sd_reg(k) = true;
                end
            end
            if any(sd_reg)
                BW2 = ismember(labelmatrix(cc), find(sd_reg));
                Ilayers(:,:,2) = BW2*grayvalue(i);
            end
    end
    Ilayers(:,:,1) = max(Ilayers,[],3);%min(Ilayers,[],3);%
    
end

attrib_feature = Ilayers(:,:,1);

switch dual
    case 'close'
        attrib_feature = MGV - attrib_feature;
    otherwise
end

    function MomentOfInertia = moments(I)
        % Calculates the moment of inertia of a binary image, the first Hu
        % invariant moment. This moment is analogous to the moment of
        % inertia around the image's centroid, where the pixels'
        % intensities are analogous to physical density.
        
        [numrows,numcols] = size(I);
        col = (1:numrows)';
        row = 1:numcols;
        X = repmat(col,[1 numcols]);
        Y = repmat(row,[numrows 1]);
        M00 = sum(sum(I));
        M10 = sum(sum(X.*I));
        M01 = sum(sum(Y.*I));
        M20 = sum(sum((X.^2).*I));
        M02 = sum(sum((Y.^2).*I));
        mu20 = M20-M10^2/M00;
        mu02 = M02-M01^2/M00;
        eita20 = mu20/(M00^2);
        eita02 = mu02/(M00^2);
        MomentOfInertia = eita20+eita02;
    end
end
